
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

# require 'rubygems'
# require 'bundler/setup'

module Tomte
  module Protocol

    LIBRARY =  File.expand_path('../..', __FILE__)
    $:.unshift LIBRARY

    begin
      require 'tomte'
    rescue LoadError
      require File.expand_path('../../../../tomte-core/lib/tomte', __FILE__)
    end

    require 'tomte/protocol/version'
    require 'amqp'

    PROTOCOL_DEFAULTS = {
      :serialization => :bson
    }.freeze

    Config.add :protocol, PROTOCOL_DEFAULTS
    Config.add_config_files [
      {:file => 'tomte-protocol.yml', :path => Configuration.relative_cfg(Tomte::Protocol::LIBRARY), :queue => :before },
      {:file => 'tomte-protocol.yml', :path => :system }
    ]

    Config.additional_options "Tomte Protocol Options"  do |config, opts|
        opts.on("--trace-protocol", "Output verbose trace of the protocol on debug level") do |t|
          config[:protocol][:trace] = true
        end
    end

    def self.configure! *args
      Tomte.configure! *args
    end


    require 'tomte/protocol/exceptions'

    autoload :Serializer,       'tomte/protocol/serializer'
    autoload :Enveloper,        'tomte/protocol/enveloper'

    autoload :Endpoint,         'tomte/protocol/endpoint'

    autoload :Channel,          'tomte/protocol/channel'
    autoload :Exchange,         'tomte/protocol/exchange'
    autoload :Queue,            'tomte/protocol/queue'

    autoload :Dialog,           'tomte/protocol/dialog'
    autoload :RPC,              'tomte/protocol/rpc'

    autoload :Synchronization,  'tomte/protocol/synchronization'

    autoload :Entity,           'tomte/protocol/entities/entity'
    autoload :Producer,         'tomte/protocol/entities/producer'
    autoload :Consumer,         'tomte/protocol/entities/consumer'

    module  Async
      autoload :Consumer,       'tomte/protocol/async/consumer'
      autoload :Callbacks,      'tomte/protocol/async/callbacks'

    end

    module Sync
      autoload :Entity,         'tomte/protocol/sync/entity'
      autoload :Producer,       'tomte/protocol/sync/producer'
      autoload :Consumer,       'tomte/protocol/sync/consumer'
      autoload :Dialog,         'tomte/protocol/sync/dialog'
      autoload :RPC,            'tomte/protocol/sync/rpc'

    end
  end

  autoload :Message, 'tomte/message'
  #autoload :MM, 'tomte/messages/mm'
end

