
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte::Protocol
  module Dialog
    class Server < Tomte::Protocol::Consumer

      class Request

        attr_accessor :metadata
        attr_accessor :payload
        attr_reader   :nak
        attr_reader   :reject


        attr_accessor :response_callback
        attr_accessor :update_callback

        alias_method :async_callback, :response_callback
        alias_method :async_callback=, :response_callback=

        def initialize(metadata, payload, &block)
          @metadata, @payload = metadata, payload

          if metadata.headers.fetch('tomte-dlg-options', []).include?('updates')
            @updates = true
          end

          @response_callback = block
          @nak = false
        end

        def wants_updates?
          !!@updates
        end

        def response
          @response ||= Response.new
        end

        def update(value, metadata = {})
          if wants_updates?
            @update_callback.call(self, value, metadata) if @update_callback
          end
        end

        def return(value)
          @response_callback.call(self, value) if @response_callback
        end

        def error(exception)
          self.return exception
        end

        def response_is_valid?
          @response.valid?
        end

        #
        # calling this causes the request not to be acked
        #
        def nak!(response_still_valid = false)
          @nak = true
          @response.valid = response_still_valid
        end

        #
        # calling this causes the request to be rejected
        # reject!(:requeue => true) asks for the request to be requeued
        #
        def reject!(opts = {})
          @reject = opts
          @response.valid = false
        end

        #
        # same as reject(:requeue => true)
        #
        def requeue!
          reject(:requeue => true)
        end


        #
        # returns a struct with sender informations
        #
        def sender
          @sender ||= Tomte::OpenDocument.new(metadata.headers.blank? ? {} :
                                                  metadata.headers.fetch('sender', {}))
        end
      end
    end
  end
end
