
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.


module Tomte::Protocol::Endpoint

  #
  # = AMQP bus adapter
  #
  class Sync
    include Tomte::Protocol::Synchronization
    extend Forwardable
    include Entities

    def_delegators :@async_endpoint,
        :identity, :name, :log, :enveloper, :enveloper=,
        :client, :options,
        :settings,
        :connected?,
        :trace!

    def initialize(options = {})
      @async_endpoint = options[:async_endpoint] || Tomte::Protocol::Endpoint::Async.new(options)
    end

    def new_channel(options = {})
      resync do |wc|
        @async_endpoint.new_channel(options) { |ch, open_ok| wc.return ch }
      end
    end

    def connect!
      resync do |wc|
        @async_endpoint.connect! { |ep| wc.return self }
      end
      self
    end

    def disconnect!
      resync do |wc|
        @async_endpoint.disconnect! { wc.return self }
      end
      self
    end

    def async
      @async_endpoint
    end

    def sync?
      true
    end

    def to_s
      "#{self.class}:[#{@async_endpoint.name} (Id: #{@async_endpoint.identity})]"
    end
  end
end