# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte::Protocol

  #
  # A null enveloper that just passes data around untouched
  # sets the headers and traces if required
  #
  module Enveloper

    # the null enveloper just passes data the way they are
    #
    class Null

      include Utils

      attr_accessor :log, :trace

      def initialize(options = {})


        @log = options[:log] || Tomte.logger
        @trace = Tomte.config.protocol[:trace] || false

        log.debug "- Using the null enveloper"
        log.warn  "= Message tracing is enabled, this will severely impact performance.".important! if trace?

      end

      #
      # converts the local representation of al object to the representation we intend to use on the wire
      #
      def local_to_wire(message, options = {})
        response = WireResponse.new(message, options, message)
        fill_in_local_headers(response)

        trace_log(:send, :metadata => response.metadata,
                         :payload => response.payload) if trace?

        response
      end

      #
      # converts the received blob into the local representation of an object
      #
      def wire_to_local(body, headers, metadata= {})

        response = LocalResponse.new

        # for tests, we return an OpenStruct if the headers is not a
        # AMQP::Headers
        response.metadata = (headers.is_a?(Hash) ? OpenStruct.new(headers) : headers)
        response.payload = body

        trace_log(:receive, :attributes => (response.metadata.attributes.nil? ?
                                            response.metadata :
                                            response.metadata.attributes),
                            :payload => response.payload,
                            :metadata => metadata) if trace?
        response
      end
    end
  end
end
