
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte::Protocol::Sync
  class Consumer
    include Tomte::Protocol::Synchronization
    include Tomte::Protocol::Sync::Entity

    attr_reader :entity

    def initialize(connection, options = {})

      initialize_from_async_entity connection
      @entity ||= Tomte::Protocol::Consumer.new(connection, options)
    end

    def consume
      resync do |wc|
        @entity.consume { wc.return }
      end
    end


    #
    # fetch a message on the queue, block until there's a message
    #
    def fetch(options = {})
      resync do |wc|

        @entity.on_delivery do |metadata, payload|
          @entity.cancel do
            wc.return_raise(payload) if payload.is_a?(Exception)
            wc.return(
              options[:metadata] ?
                Tomte::Protocol::Enveloper::LocalResponse.new(payload, metadata) :
                payload)
          end
        end

        @entity.consume(options)
      end
    end


    #
    # pop a message from the queue, returns nil if the queue is empty
    #
    def pop(options = {})
      resync do |wc|
        @entity.pop(options) do |metadata, payload|
          wc.return_raise(payload) if payload.is_a?(Exception)
          wc.return(
            options[:metadata] ?
              Tomte::Protocol::Enveloper::LocalResponse.new(payload, metadata) :
              payload)
        end
      end
    end

    def exchange(name, type = :direct, options = {})
      resync do |wc|
        @entity.exchange(name, type, options) { |ex| wc.return ex }
      end
    end

    def bind(exchange, options = {})
      resync do |wc|
        @entity.bind(exchange, options) { wc.return self }
      end
    end

    def unbind(exchange, options = {})
      resync do |wc|
        @entity.unbind(exchange, options) { wc.return self }
      end
    end

    def delete_queue(options = {})
      resync do |wc|
        @entity.delete_queue(options) { wc.return self }
      end
    end

  end
end