module Tomte::Protocol

  class Entity

    def new_sync_entity
      self.class.matching_sync_class.new(self)
    end

    def self.async_entity_name
      name.dup.tap { |s| s.slice!('Tomte::Protocol::')}
    end

    def self.matching_sync_class
      ['Tomte::Protocol::Sync', async_entity_name].join('::').safe_constantize
    end
  end

  module Sync
    module Entity
      extend Forwardable
      extend ActiveSupport::Concern

      def_delegators :@entity,
                     :connected?,
                     :enveloper,
                     :enveloper=,
                     :connection,
                     :log

      DEFAULT_REPLY_TEXT = "Goodbye.".freeze

      module ClassMethods


      end


      def connect!(options = {})

        resync do |wc|
          options.merge!(:errback => proc { |ch, channel_close|
            wc.return Tomte::Protocol::ProtocolFailure.new(
                          "Transport failure while connecting:", :transport_exception => channel_close)
          })
          @entity.connect!(options) { |entity| wc.return self }
        end
      end

      def close(reply_code = 200, reply_text = DEFAULT_REPLY_TEXT, &block)

        resync do |wc|
          @entity.close(reply_code, reply_text) { |entity| wc.return self }
        end
      end

      private

      def initialize_from_async_entity(connection_or_entity)
        if connection_or_entity.is_a?(Tomte::Protocol::Entity) && same_async_entity_class?(connection_or_entity)
          @entity = connection_or_entity
        end
      end

      def same_async_entity_class?(async_entity)
        klass = async_entity.class.matching_sync_class
        raise ArgumentError, "Cannot init a sync entity of type '#{self.class.name}' from an entity of type '#{klass.name}'" unless
          (self.class == klass)
        true
      end

    end
  end
end